$(function () {
    function ToDoList() {

        function Todo(dueDate, text, done) {
            this.id = uniqueID++;
            this.dueDate = dueDate;
            this.text = text;
            this.done = ko.observable(done);
        }

        var self = this,
            uniqueID = 0,
            changeOnStage = false,
            changeDay = true,
            d1 = new Date(),
            d2 = new Date(d1.getTime() + 1000 * 60 * 60 * 24 * 4),
            hasStorage = typeof (Storage) !== "undefined",
            todos = {};

        // Set hour information to 0 to have a
        // timestamp of the beginning of the day
        d1.setHours(0, 0, 0, 0);
        d2.setHours(0, 0, 0, 0);

        todos[d1] = [
            { dueDate: d1, text: "Check Mobiscroll demos", done: true },
            { dueDate: d1, text: "Feed the pets", done: false },
            { dueDate: d1, text: "Send an invitation email to Emily", done: false },
            { dueDate: d1, text: "Watch the new Arrow episode", done: false },
            { dueDate: d1, text: "Don't forget Margaret's birthday", done: true }
        ];

        todos[d2] = [
            { dueDate: d2, text: "Wash the car", done: false },
            { dueDate: d2, text: "Feed the pets", done: false },
            { dueDate: d2, text: "Call dad", done: false },
            { dueDate: d2, text: "Buy flowers", done: false }
        ];

        self.markedDays = [];
        self.selectedDate = ko.observable(d1);
        self.theme = ko.observable(MS.theme);

        // Observable properties
        self.isAddFormVisible = ko.observable(false);
        self.toDoText = ko.observable('');
        self.toDoTextFocus = ko.observable(false);

        // Load todo-s from localStorage
        if (hasStorage) {
            var tempList = [],
                tempObj = {};

            todos = localStorage.todos ? JSON.parse(localStorage.todos) : todos;
            $.each(todos, function (i, v) {
                $.each(v, function (index, todo) {
                    tempList.push(new Todo(new Date(todo.dueDate), todo.text, todo.done));
                });
                self.markedDays.push(new Date(i));
                tempObj[i] = tempList;
                tempList = [];
            });
            todos = tempObj;
        }

        self.currentList = ko.observableArray(todos[self.selectedDate()]);

        self.isListEmpty = ko.computed(function () {
            return !self.currentList() || !self.currentList().length;
        });

        // Change on day select
        self.onDaySelect = function (d, inst) {
            if (changeDay) {
                self.selectedDate(d.date);
                self.currentList(todos[self.selectedDate()]);
                self.isAddFormVisible(false);
            } else {
                return false;
            }
        };

        // Add a todo
        self.addToDo = function () {
            if (self.toDoText()) {
                if (!todos[self.selectedDate()]) {
                    todos[self.selectedDate()] = [];
                    self.currentList(todos[self.selectedDate()]);
                    self.markedDays.push(self.selectedDate());
                    mobiscroll.instances['todo-calendar'].refresh();
                }

                self.currentList.unshift(new Todo(self.selectedDate(), self.toDoText(), false));
                self.isAddFormVisible(false);
                self.save();
            }
            return false;
        };

        // Remove a todo
        self.removeToDo = function (event, inst) {
            self.currentList.splice(event.index, 1);
            if (self.currentList().length == 0) {
                $.each(self.markedDays, function (i, d) {
                    if (d.getTime() == self.selectedDate().getTime()) {
                        self.markedDays.splice(i, 1);
                        return false;
                    }
                });
                delete todos[self.selectedDate()];
                mobiscroll.instances['todo-calendar'].refresh();
            }
            self.save();
            return false;
        };

        // If the swiping is in the COMPLETE stage, the todo will be completed temporary
        self.stageChange = function (event, inst) {
            changeDay = false;
            if (event.stage.percent == 20 && !self.currentList()[event.index].done()) {
                changeOnStage = true;
                self.currentList()[event.index].done(true);
            } else if (event.stage.percent == 0 && changeOnStage) {
                self.currentList()[event.index].done(false);
            }
        };

        // Complete a todo
        self.checkToDo = function (event, inst) {
            if (changeOnStage || !self.currentList()[event.index].done()) {
                self.currentList()[event.index].done(true);
                self.save();
                changeOnStage = true;
                return self.currentList().length - 1 == event.index;
            }
        };

        // onItemTap event sets the todo done property to false
        self.unCheckToDo = function (event, inst) {
            self.currentList()[event.index].done(false);
            self.save();
        };

        // When a todo is done, that will be moved to the bottom of the list
        self.moveDown = function (event, inst) {
            if (changeOnStage && self.currentList().length - 1 != event.index && self.currentList()[event.index].done()) {
                inst.move(event.target, self.currentList().length - 1, null, function () {
                    self.sortUpdate.call(this, event.target, event.index);
                });
            }
            changeDay = true;
            changeOnStage = false;
        };

        self.sortStart = function (event, inst) {
            changeDay = false;
        };

        self.sortStop = function (event, inst) {
            changeDay = true;
        };

        // Returns true/false to confirm to delete
        self.confirmDelete = function (event, inst) {
            return !self.currentList()[event.index].done();
        };

        // Update todo order if order was changed on the UI
        self.sortUpdate = function (item, index, inst) {
            var list = this;

            self.currentList.sort(function (left, right) {
                var i1 = $('li', list).index($('li[data-id="' + left.id + '"]', list)),
                    i2 = $('li', list).index($('li[data-id="' + right.id + '"]', list));

                return i1 - i2;
            });
            self.save();
        };

        // Show/hide add new todo
        self.toggleAddForm = function () {
            self.isAddFormVisible(!self.isAddFormVisible());
            self.toDoText('');
            if (self.isAddFormVisible()) {
                self.toDoTextFocus(true);
            }
        };

        // Save data in local storage
        self.save = function () {
            if (hasStorage) {
                var todo = [], a = {};

                $.each(todos, function (i, v) {
                    $.each(v, function (index, t) {
                        todo.push({ text: t.text, dueDate: t.dueDate, done: t.done() });
                    });
                    a[i] = todo;
                    todo = [];
                });
                localStorage.todos = JSON.stringify(a);
            }
        };

        self.changeTheme = function (theme) {
            if (self.theme() != theme) {
                self.theme(theme);
                mobiscroll.instances['todo-calendar'].option({ theme: theme });
                mobiscroll.instances['todo-form'].option({ theme: theme });
                mobiscroll.instances['todo-listview'].option({ theme: theme });
            }
        };

        $('#theme').change(function () {
            self.changeTheme($(this).val());
        });

    }

    ko.applyBindings(new ToDoList());

});

$(window).on('load', function () {
    $('.wrapper').removeClass('loading');
});
